<?php					// -*-c++-*-
	include('xmlrpc.inc');
	require 'phpunit.php';

	$DEBUG = 0;
	$LOCALSERVER = 'xmlrpc.heddley.com';
	$HTTPSSERVER = 'xmlrpc.usefulinc.com';
	$URI  = '/server.php';
	$PATH = '/var/www/xmlrpc';

	$suite = new TestSuite;

	class TestLocalhost extends TestCase
	{
		function TestLocalhost($name)
		{
			$this->TestCase($name);
		}

		function setUp()
		{
			global $DEBUG, $LOCALSERVER, $URI;
			$this->client=new xmlrpc_client($URI, $LOCALSERVER, 80);
			if ($DEBUG)
			{
				$this->client->setDebug(1);
			}
		}

		function stringTest()
		{
			$sendstring="here are some \"entities\" < > & and " .
				"here's a dollar sign \$pretendvarname and a backslash too " . chr(92) . 
				" - isn't that great? \\\"hackery\\\" at it's best " .
				" also don't want to miss out on \$item[0]";
			$f=new xmlrpcmsg('examples.stringecho', array(
				new xmlrpcval($sendstring, 'string')
			));
			$r=$this->client->send($f);
			$v=$r->value();
			$this->assertEquals($sendstring, $v->scalarval());
		}

		function addingDoublesTest()
		{
			// note that rounding errors mean i
			// keep precision to sensible levels here ;-)
			$a=12.13; $b=-23.98;
			$f=new xmlrpcmsg('examples.addtwodouble',array(
				new xmlrpcval($a, 'double'),
				new xmlrpcval($b, 'double')
			));
			$r=$this->client->send($f);
			$v=$r->value();
			$this->assertEquals($a+$b,$v->scalarval());
		}

		function addingTest()
		{
			$f=new xmlrpcmsg('examples.addtwo',array(
				new xmlrpcval(12, 'int'),
				new xmlrpcval(-23, 'int')
			));
			$r=$this->client->send($f);
			$v=$r->value();
			$this->assertEquals(12-23, $v->scalarval());
		}

		function invalidNumber()
		{
			$f=new xmlrpcmsg('examples.addtwo',array(
				new xmlrpcval('fred', 'int'),
				new xmlrpcval("\"; exec('ls')", 'int')
			));
			$r=$this->client->send($f);
			$v=$r->value();
			// TODO: a fault condition should be generated here
			// by the server, which we pick up on
			$this->assertEquals(0, $v->scalarval());
		}

		function booleanTest()
		{
			$f=new xmlrpcmsg('examples.invertBooleans', array(
				new xmlrpcval(array(
					new xmlrpcval(true, 'boolean'),
					new xmlrpcval(false, 'boolean'),
					new xmlrpcval(1, 'boolean'),
					new xmlrpcval(0, 'boolean'),
					new xmlrpcval('true', 'boolean'),
					new xmlrpcval('false', 'boolean')
				), 
				'array'
			)));
			$answer='010101';
			$r=$this->client->send($f);
			$this->assert(!$r->faultCode());
			$v=$r->value();
			$sz=$v->arraysize();
			$got='';
			for($i=0; $i<$sz; $i++)
			{
				$b=$v->arraymem($i);
				if($b->scalarval())
				{
					$got.='1';
				}
				else
				{
					$got.='0';
				}
			}
			$this->assertEquals($answer, $got);
		}

		function base64Test()
		{
			$sendstring='Mary had a little lamb,
Whose fleece was white as snow,
And everywhere that Mary went
the lamb was sure to go.

Mary had a little lamb
She tied it to a pylon
Ten thousand volts went down its back
And turned it into nylon';
			$f=new xmlrpcmsg('examples.decode64',array(
				new xmlrpcval($sendstring, 'base64')
			));
			$r=$this->client->send($f);
			$v=$r->value();
			$this->assertEquals($sendstring, $v->scalarval());
		}

		function countEntities()
		{
			$sendstring = "h'fd>onc>>l>>rw&bpu>q>e<v&gxs<ytjzkami<";
			$f = new xmlrpcmsg('validator1.countTheEntities',array(
				new xmlrpcval($sendstring, 'string')
			));
			$r = $this->client->send($f);
			$v = $r->value();

			$got = '';
			$expected = '37210';
			$expect_array = array('ctLeftAngleBrackets','ctRightAngleBrackets','ctAmpersands','ctApostrophes','ctQuotes');

			while(list(,$val) = each($expect_array))
			{
				$b = $v->structmem($val);
				$got .= $b->me['int'];
			}

			$this->assertEquals($expected, $got);
		}

		function _multicall_msg($method, $params)
		{
			$struct['methodName'] = new xmlrpcval($method, 'string');
			$struct['params'] = new xmlrpcval($params, 'array');
			return new xmlrpcval($struct, 'struct');
		}
		
				
		function testServerMulticall()
		{
			// We manually construct a system.multicall() call to ensure
			// that the server supports it.
			
			// Based on http://xmlrpc-c.sourceforge.net/hacks/test_multicall.py
			$good1 = $this->_multicall_msg(
				'system.methodHelp',
				array(xmlrpc_encode('system.listMethods')));
			$bad = $this->_multicall_msg(
				'test.nosuch',
				array(xmlrpc_encode(1), xmlrpc_encode(2)));
			$recursive = $this->_multicall_msg(
				'system.multicall',
				array(new xmlrpcval(array(), 'array')));
			$good2 = $this->_multicall_msg(
				'system.methodSignature',
				array(xmlrpc_encode('system.listMethods')));
			$arg = new xmlrpcval(
				array($good1, $bad, $recursive, $good2),
				'array');
			
			$f = new xmlrpcmsg('system.multicall', array($arg));
			$r = $this->client->send($f);
			$this->assert($r->faultCode() == 0, "fault from system.multicall");

			$v = $r->value();
			$this->assert($v->arraysize() == 4, "bad number of return values");

			$r1 = $v->arraymem(0);
			$this->assert(
				$r1->kindOf() == 'array' && $r1->arraysize() == 1,
				"did not get array of size 1 from good1");

			$r2 = $v->arraymem(1);
			$this->assert(
				$r2->kindOf() == 'struct',
				"no fault from bad");

			$r3 = $v->arraymem(2);
			$this->assert(
				$r3->kindOf() == 'struct',
				"recursive system.multicall did not fail");

			$r4 = $v->arraymem(3);
			$this->assert(
				$r4->kindOf() == 'array' && $r4->arraysize() == 1,
				"did not get array of size 1 from good2");
		}

		function testClientMulticall()
		{
			$good1 = new xmlrpcmsg('system.methodHelp',
				array(xmlrpc_encode('system.listMethods')));
			$bad = new xmlrpcmsg('test.nosuch',
				array(xmlrpc_encode(1), xmlrpc_encode(2)));
			$recursive = new xmlrpcmsg('system.multicall',
				array(new xmlrpcval(array(), 'array')));
			$good2 = new xmlrpcmsg('system.methodSignature',
				array(xmlrpc_encode('system.listMethods')));

			$r = $this->client->send(array($good1, $bad, $recursive, $good2));

			$this->assert(count($r) == 4, "wrong number of return values");

			$this->assert($r[0]->faultCode() == 0, "fault from good1");
			$val = $r[0]->value();
			$this->assert(
				$val->kindOf() == 'scalar' && $val->scalartyp() == 'string',
				"good1 did not return string");
			$this->assert($r[1]->faultCode() != 0, "no fault from bad");
			$this->assert($r[2]->faultCode() != 0, "no fault from recursive system.multicall");
			$this->assert($r[3]->faultCode() == 0, "fault from good2");
			$val = $r[3]->value();
			$this->assert($val->kindOf() == 'array', "good2 did not return array");


			// This is the only assert in this test which should fail
			// if the test server does not support system.multicall.
			$this->assert($this->client->no_multicall == false,
				"server does not support system.multicall");
		}
	}

	class TestFileCases extends TestCase
	{
		function TestFileCases($name, $base='')
		{
			if(!$base)
			{
				global $PATH;
				$base = $PATH;
			}
			$this->TestCase($name);
			$this->root=$base;
		}

		function stringBug ()
		{
			$m=new xmlrpcmsg('dummy');
			$fp=fopen($this->root.'/bug_string.xml', 'r');
			$r=$m->parseResponseFile($fp);
			$v=$r->value();
			fclose($fp);
			$s=$v->structmem('sessionID');
			$this->assertEquals('S300510007I', $s->scalarval());
		}

		function whiteSpace ()
		{
			$m=new xmlrpcmsg('dummy');
			$fp=fopen($this->root.'/bug_whitespace.xml', 'r');
			$r=$m->parseResponseFile($fp);
			$v=$r->value();
			fclose($fp);
			$s=$v->structmem('content');
			$this->assertEquals("hello world. 2 newlines follow\n\n\nand there they were.", $s->scalarval());
		}
	}

	class TestInvalidHost extends TestCase
	{
		function TestInvalidHost($name)
		{
			$this->TestCase($name);
		}

		function setUp()
		{
			global $DEBUG,$LOCALSERVER;
			$this->client=new xmlrpc_client('/NOTEXIST.php', $LOCALSERVER, 80);
			if($DEBUG)
			{
				$this->client->setDebug(1);
			}
		}

		function test404()
		{
			$f=new xmlrpcmsg('examples.echo',array(
				new xmlrpcval('hello', 'string')
			));
			$r=$this->client->send($f);
			$this->assertEquals(5, $r->faultCode());
		}
	}

	class TestHTTPSConnection extends TestCase
	{
		function TestInvalidHost($name)
		{
			$this->TestCase($name);
		}

		function setUp()
		{
			global $DEBUG,$HTTPSSERVER,$URI;
			$this->client=new xmlrpc_client($URI,$HTTPSSERVER);
			//$this->client->setCertificate('/var/www/xmlrpc/rsakey.pem',
			//			  'test');
			if ($DEBUG || 1)
			{
				$this->client->setDebug(1);
			}
		}

		function addingTest()
		{
			$f=new xmlrpcmsg('examples.getStateName',array(
				new xmlrpcval(23, 'int')
			));
			$r=$this->client->send($f, 180, 'https');
			if($r->faultCode())
			{
				// create dummy value so assert fails
				$v=new xmlrpcval('SSL send failed.');
				print '<pre>Fault: ' . $r->faultString() . '</pre>';
			}
			else
			{
				$v=$r->value();
			}
			$this->assertEquals('Michigan', $v->scalarval());
		}
	}

	$suite->addTest(new TestLocalhost('stringTest'));
	$suite->addTest(new TestLocalhost('addingTest'));
	$suite->addTest(new TestLocalhost('addingDoublesTest'));
	$suite->addTest(new TestLocalhost('invalidNumber'));
	$suite->addTest(new TestLocalhost('booleanTest'));
	$suite->addTest(new TestLocalhost('countEntities'));
	$suite->addTest(new TestLocalhost('base64Test'));
	$suite->addTest(new TestLocalhost('testServerMulticall'));
	$suite->addTest(new TestLocalhost('testClientMulticall'));
	$suite->addTest(new TestInvalidHost('test404'));
	$suite->addTest(new TestFileCases('stringBug'));
	$suite->addTest(new TestFileCases('whiteSpace'));
	$suite->addTest(new TestHTTPSConnection('addingTest'));
	$title = 'XML-RPC Unit Tests';
?>
<html>
<head><title><?php echo $title; ?></title></head>
<body>
<h1><?php echo $title; ?></h1>
<p>Note, tests beginning with 'f_' <i>should</i> fail.</p>
<p>
<?php
	if (isset($only))
	{
		$suite = new TestSuite($only);
	}

	$testRunner = new TestRunner;
	$testRunner->run($suite);
?>
</body>
</html>
